%{
// Simple lexer for sample calculator

/////////////////////////////////////////////////////////////////////
// Declarations section

// Compile with -DLEXDEBUG to get a stand-alone test
#ifdef LEXDEBUG
#include <iostream>	// For debugging
using namespace std;

// make up an attribute type for testing. In normal use,
// this structure is provided in the "simple.tab.h" file
// from the bison output.
typedef union
{
	int intval;
} YYSTYPE;
#define INTEGER 256

// Instantiate the global variable from bison. This is also
// normally part of simple.tab.h.
YYSTYPE yylval;

#else
// Use it for real (i.e. not just testing lexer)
#include "simple.tab.h"	// definitions of token encodings

#endif

#include <cstdlib>	// For atoi()
using namespace std;

// Externally known symbols.
// Required by flex--
extern "C" int yywrap ( void );	// required by lex
extern YYSTYPE yylval;	// the secondary lexer return value
// Our own choice--
int LineNo = 1;		// count source lines

// End of  C++ declarations section
/////////////////////////////////////////////////////////////////////
%}

INT	([0-9]+)
WS	([ \t]+)

%start comment

%%
\n	{	// End of line; count lines
		LineNo++;
	}

<comment>"*/"	{	// end of comment
		BEGIN ( INITIAL );
	}

<comment>.	{	// any other character in a comment
		// do nothing
	}

<INITIAL>"/*"	{	// beginning of comment
		BEGIN ( comment );
	}

<INITIAL>{INT}	{	// String of digits
		yylval.intval = atoi( yytext ); // Attribute = integer's value
		return INTEGER; // Token type is INTEGER
	}

<INITIAL>{WS}	{	// White space; Do nothing
	}

<INITIAL>.	{	// Anything else (e.g. '+', ';'
				// note no yylval is needed or set
		return *yytext; // These simple token types are self-defining
	}

%%

/////////////////////////////////////////////////////////////////////
// Definitions section

// There are no include files; EOF is really end of file!
int yywrap()
{
	return 1;
}

#ifdef LEXDEBUG
int main()
{
	int token;
	while ( token = yylex() )
	{
		cout << token;
		if ( token < 256 ) cout << "(" << char(token) << ")";
		cout << endl;
	}
}
#endif

